-- Copyright (C) 2021  Dakra-SilverHand
-- SPDX-License-Identifier: GPL-3.0-only

local function FullName(name)
   if not name:find('-') then
      local realm = GetRealmName()
      realm = realm:gsub(' ', '')
      name = name..'-'..realm
   end
   return name
end

local GUILD_NAME
local GUILD_ROSTER_INFO_BY_NAME
local function GuildRosterInfoByName(findName)
   if not GUILD_ROSTER_INFO_BY_NAME then
      GUILD_NAME = GetGuildInfo('player')
      GUILD_ROSTER_INFO_BY_NAME = {}
      for i = 1,10000 do
         local t = {GetGuildRosterInfo(i)}
         local curName = t[1]
         if not curName then break end
         GUILD_ROSTER_INFO_BY_NAME[curName] = t
         --print('+', curName)
      end
   end
   local fullName = FullName(findName)
   return GUILD_ROSTER_INFO_BY_NAME[fullName]
end

--

---@param text string @The format string like "Greetings %s! How are you?"
---@return string @Returns a pattern like "Greetings (.-)%! How are you%?"
local function FormatToPattern(text)
   if type(text) ~= "string" then return end
   text = text:gsub("%%", "%%%%")
   text = text:gsub("%.", "%%%.")
   text = text:gsub("%?", "%%%?")
   text = text:gsub("%+", "%%%+")
   text = text:gsub("%-", "%%%-")
   text = text:gsub("%(", "%%%(")
   text = text:gsub("%)", "%%%)")
   text = text:gsub("%[", "%%%[")
   text = text:gsub("%]", "%%%]")
   text = text:gsub("%%%%s", "(.-)")
   text = text:gsub("%%%%d", "(%%d+)")
   text = text:gsub("%%%%%%[%d%.%,]+f", "([%%d%%.%%,]+)")
   return text
end

local FORMAT_GUILD = "^" .. FormatToPattern(WHO_LIST_GUILD_FORMAT) .. "$"
local FORMAT = "^" .. FormatToPattern(WHO_LIST_FORMAT) .. "$"

--

local function EventFilter(self, event, text, ...)
   if event ~= "CHAT_MSG_SYSTEM" then return false end

   local nameLink, name, level, race, class, guild, zone = text:match(FORMAT_GUILD)
   if not nameLink then return false end

   if not zone then
      guild = nil
      nameLink, name, level, race, class, zone = text:match(FORMAT)
   end
   if not nameLink or not level then return false end

   --

   local gri = GuildRosterInfoByName(name)
   if not gri then return end
   local _, rankName, _, _, _, _, publicNote, officerNote = unpack(gri)
   if officerNote == '' then -- Why is it like this.
      officerNote = nil
   end

   local gstr = string.format('%s (%s) "%s"', GUILD_NAME, rankName, publicNote)
   if officerNote then
      gstr = string.format('%s (%s)', gstr, officerNote)
   end

   local br_gname_br = string.format('<%s>', GUILD_NAME)
   local br_gstr_br = string.format('<%s>', gstr)
   local text2 = text:gsub(br_gname_br, br_gstr_br) -- Definitely safe and reliable! :P
   return false, text2, ...
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", EventFilter)
